//
// Copyright (c) 2002-2023 WEINZIERL ENGINEERING GmbH
// All rights reserved.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY,
// WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE
//

#if !defined USE_POCO_LEGACY_LOGGER || USE_POCO_LEGACY_LOGGER != 1


#ifndef __WZCPP_LOGGING_WZLOGGER_H__
#define __WZCPP_LOGGING_WZLOGGER_H__

#include "LoggerInterface.h"
#include <spdlog/sinks/rotating_file_sink.h>
#include <spdlog/spdlog.h>
#include <memory>


namespace wzcpp
{

class WzLogger : public BaseLogger
{
public:
	using BaseLogger::BaseLogger;

	WzLogger(const std::string& name, LogLevel level) : BaseLogger(name, level)
	{
		init();
	}

	void init() override
	{
		if (getLogger() == nullptr)
		{
			// If loglevel is lower than error the output is redirected to stdout
			if (logLevel_ < LogLevel::error) getLogger() = spdlog::stdout_color_st(name_);
			// If loglevel is at least error the output is redirected to stderr
			else
				getLogger() = spdlog::stderr_color_st(name_);

			// set level
			getLogger()->set_level(spdlog::from_wzlevel(logLevel_));
		}
	}

	void setLevel(LogLevel level) override
	{
		BaseLogger::setLevel(level);
		getLogger()->set_level(spdlog::from_wzlevel(level));
	}

	void verbose(const std::string& msg) override
	{
		log(LogLevel::verbose, msg);
	}

	void debug(const std::string& msg) override
	{
		log(LogLevel::debug, msg);
	}

	void info(const std::string& msg) override
	{
		log(LogLevel::info, msg);
	}

	void warn(const std::string& msg) override
	{
		log(LogLevel::warn, msg);
	}

	void error(const std::string& msg) override
	{
		log(LogLevel::error, msg);
	}

	void critical(const std::string& msg) override
	{
		log(LogLevel::critical, msg);
	}

	void addLoggingChannel(std::shared_ptr<spdlog::sinks::sink> sink) override
	{
		getLogger()->sinks().emplace_back(sink);
	}

	void removeLoggingChannel(std::shared_ptr<spdlog::sinks::sink> sink) override
	{
		auto sinks = getLogger()->sinks();
		sinks.erase(std::remove(sinks.begin(), sinks.end(), sink), sinks.end());
	}

private:
	void log(LogLevel lvl, const std::string& msg)
	{
        auto logger = spdlog::get(name_);
		logger->log(spdlog::from_wzlevel(lvl), msg + "\n");
        logger->flush();

	}

	std::shared_ptr<spdlog::logger> getLogger()
	{
		return spdlog::get(name_);
	}
};

}  // namespace wzcpp
#endif  // !__WZCPP_LOGGING_WZLOGGER_H__


#endif  // !USE_POCO_LEGACY_LOGGER
