//
// Copyright (c) 2021 WEINZIERL ENGINEERING GmbH
// All rights reserved.
//
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
// IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
// FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
// SHALL THE COPYRIGHT HOLDERS BE LIABLE FOR ANY DAMAGES OR OTHER LIABILITY,
// WHETHER IN CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
// WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE
//

////////////////////////////////////////////////////////////////////////////////
//
// This header contains functions which affect the container classes from
// the C++ standard library like std::vector, std::map, etc.
//
////////////////////////////////////////////////////////////////////////////////

#ifndef __WZCPP_UTILS_CONTAINER_H__
#define __WZCPP_UTILS_CONTAINER_H__

#include <ostream>
#include <sstream>
#include <string>
#include <type_traits>
#include <vector>


namespace wzcpp
{

/*!
    Joins the values of a string vector with a delimiter in between
    @param vect      The list of strings to join
    @param delimiter The string to put in between the values
    @returns         The joined string
*/
inline std::string join(const std::vector<std::string>& vect, const char* delimiter)
{
	std::ostringstream sstream;

	for (auto&& s : vect)
	{
		sstream << s;
		if (s != vect.back()) sstream << delimiter;
	}

	return sstream.str();
}
}  // namespace wzcpp


#endif  // __WZCPP_UTILS_CONTAINER_H__
