#include <baos/Sdk.h>
#include <baos/ip/BaosIpv4Device.h>
#include <baos/ip/BaosTcpConnection.h>

#include <iostream>
#include <memory>

/*!
    A stripped down version of the  01_BaosDeviceInformation.cpp sample
*/

using wz::baos::setBaosLogLevel;
using namespace wz::baos;

int main(int argc, char* argv[])
{
	try
	{
		std::cout << "******************************************" << std::endl;
		std::cout << "MyBaosProject sample" << std::endl;
		std::cout << "******************************************" << std::endl;
		std::cout << std::endl;

		// Reduce log messages
		setBaosLogLevel(wzcpp::LogLevel::warn);

		// Get the ip address from the parsed command line options
		const std::string ipAddress = "10.0.0.102"; // YOUR IP ADDRESS HERE
		std::cout << " Connection requested for " << ipAddress << std::endl;

		// create a TPC/IP connection with the remote BAOS device
		ip::BaosIp4Device baosDevice(ipAddress);

		// read server items with static infos
		const DeviceStaticInfo staticInfo = baosDevice.getStaticInfo();
		std::cout << "Serial number: " << staticInfo.serialNumber.toString() << std::endl;
		std::cout << "Manufacture code device: " << staticInfo.manufactureCodeDevice << std::endl;
		std::cout << "MAC address: " << staticInfo.macAddress.toString() << std::endl;
		std::cout << "Max datapoints: " << staticInfo.maxDatapoints << std::endl;
		std::cout << "Max parameter bytes: " << staticInfo.maxParameterBytes << std::endl;

		// read server items with config infos
		const DeviceConfigInfo configInfo = baosDevice.getConfigInfo();
		std::cout << "Manufacture code app: " << configInfo.manufactureCodeApp << std::endl;
		std::cout << "Application ID: " << configInfo.applicationId << std::endl;
		std::cout << "Application version: " << static_cast<int>(configInfo.applicationVersion) << std::endl;

		// read server items with runtime infos
		// everything that may change during the runtime of the device
		const DeviceRuntimeInfo runtimeInfo = baosDevice.getRuntimeInfo();
		std::cout << "KNX bus connected: " << std::boolalpha << runtimeInfo.busConnected << std::endl;
		std::cout << "Programming mode: " << std::boolalpha << runtimeInfo.programmingMode << std::endl;
	}
	catch (const wzcpp::error::WzBaseException& e)
	{
		std::cout << "Wz Exception in main: " << e.msg() << std::endl;
	}
	catch (const std::exception& e)
	{
		std::cerr << "Failed: " << e.what() << std::endl;
		return -1;
	}

	return 0;
}